#include <Arduino.h>
#include <stdint.h>
#include "parameters.h"
#include "sampler.h"
#include "autocorr.h"


float interpolate(int32_t a, int32_t b, int32_t c)
{
	return 0.5 * (c - a) / (2 * b - a - c);
}

int32_t autocorrelation(int8_t *samples, uint_least16_t t)
{
	int32_t r = 0;
	for (uint_least16_t k = 0; k < no_samples - t; k++)
		r += ((int16_t)samples[k] * samples[k + t]);
	return r;
}

float frequency(int8_t *samples)
{
	if (!samples)
		return 0;

	float p = 0;
	int32_t r = autocorrelation(samples, 0);
	int32_t target = (float)r * 0.8;
	int32_t rl = r;
	int32_t rll = r;
	uint_least8_t st = 0;

	for (uint_least16_t t = 1; (t < (no_samples / 2)); t++)
	{
		if (st == 2)
			break;

		r = (autocorrelation(samples, t) * no_samples) / (no_samples - t);

		switch (st)
		{
		case 0:
			if ((r > target) && (r > rl))
				st = 1;
			break;
		case 1:
			if (r <= rl)
			{
				st = 2;
				p = t - 1 + interpolate(rll, rl, r);
			}
			break;
		}
		rll = rl;
		rl = r;
	}

	if (st != 2)
		return 0;

	float f = (float)sample_rate / p;

	if (f < 1600)
		return f;
	else
		return 0;

}
