#include <Arduino.h>
#include <stdint.h>
#include <SPI.h>
#include <SD.h>
#include <LiquidCrystal_I2C.h>
#include <Wire.h>

#include "parameters.h"
#include "autocorr.h"
#include "sampler.h"

LiquidCrystal_I2C lcd(0x3F, 16, 2);
unsigned long elapsed;
uint8_t last_state = 0;
uint8_t current_state;
uint8_t led_state = 0;
uint8_t current_instrument = 0;
File writefile;

void setup()
{
    Serial.begin(31250);
    lcd.init();
    lcd.backlight();
    SD.begin(4);
    writefile = SD.open("freqs.txt", FILE_WRITE);
    DDRD |= (1 << PD6);
    DDRD &= ~(1 << PD7);
    PORTD |= (1 << PD6);
    PIND |= (1 << PD7);
    begin();
}

void loop()
{
    uint8_t val = 0;

    if (PIND & (1 << PD7))
        val = 1;

    if (val != last_state)
        elapsed = millis();

    if (millis() - elapsed > debounce_time)
    {
        if (val != current_state)
        {
            current_state = val;
            if (current_state == 1)
            {
                led_state = !led_state;
            }
        }
    }

    if (led_state == 1)
    {
        PORTD |= (1 << PD6);
        Serial.write(0xC0);
	    Serial.write(current_instrument & 0x7F);
        delay(100);
        current_instrument++;
        if (current_instrument == 127)
            current_instrument = 0;
        led_state = 0;
    }
    else
        PORTD &= ~(1 << PD6);
    last_state = val;

    lcd.setCursor(0, 0);
    lcd.print("Your Frequency:");

    int8_t *samples = return_samples();
    float f = frequency(samples);
    writefile.println(f);
    sample();

    if (f > 32)
    {
        lcd.setCursor(4, 1);
        lcd.print(f);
        lcd.setCursor(10, 1);
        lcd.print("Hz");
        uint8_t mult = 0;
        uint_least8_t p;
        while (f > 31.77)
        {
            f /= 2;
            mult++;
        }
        for (uint_least8_t i = 0; i < 12; i++)
        {
            if (f < notes[i].max)
            {
                p = i;
                break;
            }
        }
        p = mult * 12 + p;
        Serial.write(0x90);
	    Serial.write(p & 0x7F);
	    Serial.write(0x7F);
        delay(200);
    }
}
